#!/usr/bin/python

# Pyle, a WikiClone in Python
# Copyright (C) 1999, 2000 Tony Garnock-Jones <tonyg@kcbbs.gen.nz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

#
# This module contains the actions that the CGI program can perform.
# The routines here are invoked by pyle.cgi.
# It uses routines in Utils.py.

import sys
import shelve
import string
import os
import re
import cgi
import time
import posix

from Utils import *
import Users
import Config

def checkperms(pagename, mode, action = None, die = 1):
	if static_mode():
		return 1

	rights = dbget(pagename, 'rights', Config.default_rights)
	message = ''

	if Users.logged_in():
		if Users.superuser(): return 1

		name = Users.username()
		if name == dbget(pagename, 'creator'): return 1
		if name == dbget(pagename, 'owner'): return 1
		if rights['named ' + mode]: return 1

		if die:
			message = message + 'Access for user <b>' + name + '</b> '
	else:
		if rights['anonymous ' + mode]: return 1

		if die:
			message = message + 'Anonymous access '

	if die:
		message = message + 'denied when trying to ' + mode
		if mode == 'read':
			message = message + ' from '
			link_mode = 'backlinks'
		else:
			message = message + ' to '
			link_mode = 'read'
		message = message + 'page <b>' + internal_link(pagename, link_mode) + '</b>.<br>'

		if action:
			message = message + 'Action was: <b>' + action + '</b><br>'

		die_with_message(message)

	return 0

###########################################################################
# Handler for 'mode=read' - displays a WikiPage

def readpage(pagename, args = None, version = None):
	checkperms(pagename, 'read', 'reading page')

	if version:
		pagetitle = pagename + ' version ' + version
	else:
		pagetitle = pagename

	text = dbgettext(pagename, version)
	modtime = dbget(pagename, 'time', 0)
	moduser = dbget(pagename, 'user', '<unknown>')
	if modtime:
		timetext = ' ('
		if use_cvs:
			timetext = timetext + 'version ' + dbgettextversion(pagename) + '; '
		timetext = timetext + 'last modified ' + \
			time.strftime("%d %B %Y", time.gmtime(modtime)) + \
			' by ' + moduser + ')'
	else:
		timetext = ''

	allbacklinks = getbacklinks(pagename)
	allbacklinks = filter(lambda x: not x in [pagename,
						  'RecentChanges',
						  'NotSoRecentChanges'],
			      allbacklinks)

	if static_mode() or len(allbacklinks) == 0:
		backlinks = '<h1 class="wikipagetitle">' + \
			    internal_link(pagename, 'backlinks', pagetitle) + '</h1>'
	else:
		backlinks = ''
		backlinks = backlinks + '<form method="post" action="' + scriptname + '">'
		backlinks = backlinks + '<h1 class="wikipagetitle">' + \
			    internal_link(pagename, 'backlinks', pagetitle)
		backlinks = backlinks + '<br><font size=2>Backlinks: '
		backlinks = backlinks + '<input type="hidden" name="mode" value="read">'
		backlinks = backlinks + '<select name="arg">'

		for l in allbacklinks:
			backlinks = backlinks + '<option>' + l + '</option>'

		backlinks = backlinks + '</select>'
		backlinks = backlinks + '<input type="submit" name="submit" value=" Go ">'
		backlinks = backlinks + '</font>'
		backlinks = backlinks + '</h1></form>'

	stdheader(pagetitle, backlinks)

	render(text, pagename, args)
	if static_mode():
		print '<p><hr>%s' % \
			timetext
	else:
		if use_cvs:
			history_string = ' | ' + internal_link(pagename, 'history', 'Page History')
		else:
			history_string = ''

		subscription_link = internal_link(pagename, 'subscribe', 'Subscribe to this page')
		if Users.logged_in() and Users.current().is_subscribed_to(pagename):
			subscription_link = internal_link(pagename,
							  'unsubscribe',
							  'Unsubscribe from this page')

		print '<p><hr>%s%s<br>%s%s<br>%s<br>%s' % \
		      (internal_link(pagename, 'edit', 'Edit this page', version),
		       timetext,
		       subscription_link,
		       history_string,
		       Users.get_toolbar(),
		       internal_link(pagename, 'find', 'Search'))
	stdtrailer()

###########################################################################
# Handler for 'mode=edit' - displays a textarea and associated controls
# which allow you to edit a WikiPage

def editpage(pagename, version = None):
	checkperms(pagename, 'write', 'editing page')

	text = dbgettext(pagename, version)
	stdheader('Edit ' + pagename)
	print '<form method="post" action="' + scriptname + '">'
	print '<h1 class="wikipagetitle">' + pagename
	if version:
		print '(from version ' + version + ')'
	print '<input type="submit" value=" Save "> <input type="reset" value=" Reset "></h1>'
	print '<textarea cols=' + str(textarea_cols) + ' rows=' + str(textarea_rows) + ' name="data" wrap=virtual>'
	print cgi.escape(text) + '</textarea>'
	print '<input type="hidden" name="mode" value="save">'
	print '<input type="hidden" name="arg" value="' + pagename + '">'
	print '<hr>'
	print '<input type="checkbox" value="yes" name="convertspaces">Convert 8-spaces to tabs?<br>'
	print '<input type="submit" value=" Save "> <input type="reset" value=" Reset ">'
	print '</form>'
	print internal_link(pagename, 'read', 'Cancel changes made, and view this page as currently saved') + '<br>'
	print internal_link(pagename, 'history', 'Cancel changes made, and view the page history for this page') + '<br>'
	print internal_link(pagename, 'attach', 'Edit attachments for this page') + '<br>'
	if checkperms(pagename, 'rights', None, 0):
		print internal_link(pagename, 'rights', 'Edit permissions for this page') + '<br>'
	print Users.get_toolbar() + '<br>'
	print '<br>'
	print internal_link(pagename, 'del', 'Delete this page')
	stdtrailer()

###########################################################################
# Handler for 'mode=find' - displays the SearchPage, with whatever content
# (including [Search] forms) the user has chosen to put there.

def searchform(pattern):
	readpage("SearchPage", { 'pattern': pattern })

###########################################################################
# Handler for 'mode=search' - displays the results of a search, usually
# activated as the result of a [Search] form.

def searchpage(pattern):
	if not pattern:
		pattern = '.'
	stdheader('Search results')
	print '<h1 class="wikipagetitle">Search results (pattern = "' + pattern + '") </h1>'
	print '<ul>'
	r = re.compile(pattern, re.IGNORECASE)

	count = 0
	for k in db.keys():
		if dbpageexists(k) and checkperms(k, 'read', None, 0):
			lines = string.split(dbgettext(k), '\n')
			for line in lines:
				if r.search(line):
					print ' <li>' + internal_link(k) + ': ...' + cgi.escape(line[:60]) + '...'
					count = count + 1
					break
	print '</ul>'
	if count == 0:
		print '<p><i>No matches found</i></p>'
	stdtrailer()

###########################################################################
# Handler for 'mode=backlinks' - searches for pages which refer to a given
# WikiPage

def backlinks(pagename):
	stdheader('Backlinks for ' + pagename)
	print '<h1 class="wikipagetitle">', internal_link(pagename, 'find'), '</h1>'
	print 'These pages link to ' + internal_link(pagename) + ':<br><br>'
	l = getbacklinks(pagename)
	print '<ul>'
	for p in l: print ' <li> ' + internal_link(p)
	print '</ul>'
	stdtrailer()

###########################################################################
# Handler for 'mode=save' - usually submitted as the result of an
# editpage() previously invoked

def savepage(pagename, data, convertspaces):
	checkperms(pagename, 'write', 'saving page')

	if not data:
		die_with_message(
			"Empty pages are not allowed. Please go back and enter some text.")

	timenow = time.time()

	dbset(pagename, 'time', timenow)
	dbset(pagename, 'user', Users.username())
	dbset(pagename, 'text', preprocess(pagename, data, convertspaces))

	notechange(pagename, 'saved', timenow)
	notify_subscribers(pagename)
	redirect(pagename, "Page Saved")

###########################################################################
# Handler for 'mode=del' - removes a page from the database completely,
# after first prompting the user.

def delpage(pagename):
	checkperms(pagename, 'write', 'deleting page')

	if getarg('confirm', 'no') == 'yes':
		dbremove(pagename)
		notechange(pagename, 'removed', time.time())
		redirect(pagename, "Page Deleted")
	else:
		title = 'Delete ' + pagename + '?'
		stdheader(title, '<h1 class="wikipagetitle">' + title + '</h1>')
		print """
<p><b>WARNING:</b> You are about to delete a page from this Wiki. This
action is <b>irreversible</b>, and will not only delete the page contents,
but also all associated attachments.</p>
<p>If you wish to continue, check the checkbox below and click on the
"Confirm" button. Otherwise, use the "back" button of your browser.</p>
<center>
<form method="post" action="%(scriptname)s">
<input type="hidden" name="mode" value="del">
<input type="hidden" name="arg" value="%(pagename)s">
<input type="checkbox" name="confirm" value="yes">&nbsp;Confirm page deletion<br>
<input type="submit" value=" Confirm ">
</form>
</center>
""" % {
	'scriptname': scriptname,
	'pagename': pagename
}
		stdtrailer()

###########################################################################
# class Attachment
#
# a record for containing information about attachments associated with a
# page.

class Attachment:
	def __init__(self, pagename, name, mimetype, content, inlinecontent = 0, attrs = ''):
		if use_cvs:
			self.pagename = pagename

		self.name = name
		self.mimetype = mimetype
		self.setcontent(content)
		self.inlinecontent = inlinecontent
		self.attrs = attrs

	def setcontent(self, content):
		if use_cvs:
			db.dict[self.pagename + '.attach.' + self.name + '.txt'] = content
		else:
			self.content = content

	def get_content(self):
		if use_cvs:
			return db.dict[self.pagename + '.attach.' + self.name + '.txt']
		else:
			return self.content

	def clear(self):
		if use_cvs:
			del db.dict[self.pagename + '.attach.' + self.name + '.txt']

	def getattrs(self):
		if hasattr(self, 'attrs'):
			return self.attrs
		else:
			return ''

###########################################################################
# Handler for 'mode=attach' - shows the list of attachments available for
# a given page, and allows the user to upload a document as a new
# attachment.

def mimetypes_string(current = None):
	def mapper(tpair, current=current):
		(t, extn) = tpair
		if (t == current):
			result = '<option selected>'
		else:
			result = '<option>'
		return result + t + '</option>'

	return string.join(map(mapper, mimetypes), '')

def attachpage(pagename):
	checkperms(pagename, 'write', 'editing page attachments')

	attachments = dbget(pagename, 'attachments', {})

	stdheader('Attachments for ' + pagename)
	print '<h1 class="wikipagetitle">Attachments for ' + pagename + '</h1>'
	print 'View ' + internal_link(pagename) + '<br>'
	print 'Edit ' + internal_link(pagename, 'edit') + '<br>'
	print '<br>'
	print 'Existing attachments:<hr>'
	for name in attachments.keys():
		a = attachments[name]
		print '<h3>Attachment name: ' + a.name + '</h3>'
		print '<form method="post" enctype="multipart/form-data" action="' + scriptname + '">'
		print '<input type="hidden" value="' + pagename + '" name="arg">'
		print '<input type="hidden" value="updattach" name="mode">'
		print '<input type="hidden" value="' + a.name + '" name="data">'
		print '<ul>'
		print '<li>MIME type: <select name="mimetype">' + mimetypes_string(a.mimetype) + '</select>'
		print '<li>Extra HTML attributes: <input type="text" name="extraattrs" value="' + a.getattrs() + '">'
		print '<li>Inlinable: <input type="checkbox" name="inline" value="yes"',
		if a.inlinecontent:
			print ' checked=yes>'
		else:
			print '>'
		print '<li>Current content length: ' + str(len(a.get_content())) + '<br>'
		print '<li>Replacement content (if empty, old content is retained): <input type="file" name="content"><br>'
		print '<li><input type="submit" value=" Update ">'
		print '<li>' + internal_link(pagename, 'getattach', 'View this attachment', a.name)
		print '<li>' + internal_link(pagename, 'delattach', 'Delete this attachment', a.name)
		print '</ul>'
		print '</form>'

	if not(len(attachments)):
		print '<i>No attachments</i>'

	print '<br><br>Add new attachment:<hr>'
	print '<form method="post" enctype="multipart/form-data" action="' + scriptname + '">'
	print '<input type="hidden" value="' + pagename + '" name="arg">'
	print '<input type="hidden" value="newattach" name="mode">'
	print 'Name: <input type="text" size="40" name="data"><br>'
	print 'Extra HTML attributes: <input type="text" name="extraattrs" size="40"><br>'
	print 'MIME type: <select name="mimetype">'
	print mimetypes_string()
	print '</select><br>'
	print '<input type="checkbox" name="inline" value="yes"> Inlinable?<br>'
	print 'Content: <input type="file" name="content"><br>'
	print '<input type="submit" value=" Save ">'
	print '</form>'

	stdtrailer()

###########################################################################
# Handler for 'mode=newattach' - attaches an uploaded document to a page

def newattachpage(pagename, name, mimetype, content, inline, extraattrs):
	checkperms(pagename, 'write', 'editing page attachments')

	if not re.search(r'^[-_.\w]+$', name):
		die_with_message("Attachment name invalid. (Pattern for name: ^[-_.\\w]+$)")

	if inline:
		inline = 1

	attachments = dbget(pagename, 'attachments', {})
	a = Attachment(pagename, name, mimetype, content, inline, extraattrs)
	attachments[name] = a
	dbset(pagename, 'attachments', attachments)

	attachpage(pagename)

###########################################################################
# Handler for 'mode=updattach' - changes an attachment's details

def updattachpage(pagename, name, mimetype, content, inline, extraattrs):
	checkperms(pagename, 'write', 'editing page attachments')

	attachments = dbget(pagename, 'attachments', {})
	if not attachments.has_key(name):
		die_with_message('Attachment "' + name + '" not found for update.')

	if not content:
		content = attachments[name].get_content()

	newattachpage(pagename, name, mimetype, content, inline, extraattrs)

###########################################################################
# Handler for 'mode=getattach' - retrieves an attached document from a
# page, and returns it to the user with the associated mimetype.

def getattachpage(pagename, name):
	checkperms(pagename, 'read', 'reading page attachment')

	attachments = dbget(pagename, 'attachments', {})

	if attachments.has_key(name):
		a = attachments[name]
		sys.stdout.write('Content-type: ' + a.mimetype + '\r\n')
		sys.stdout.write('Content-disposition: attachment; filename="' + \
				 name + '"\r\n\r\n')
		sys.stdout.write(a.get_content())
	else:
		die_with_message("Cannot view attachment: unknown attachment name.")

###########################################################################
# Handler for 'mode=delattach' - removes an attachment from a page, after
# first confirming the user's intention.

def delattachpage(pagename, name):
	checkperms(pagename, 'write', 'deleting page attachment')

	if getarg('confirm', 'no') == 'yes':
		attachments = dbget(pagename, 'attachments', {})

		if attachments.has_key(name):
			attachments[name].clear()
			del attachments[name]
			dbset(pagename, 'attachments', attachments)
			attachpage(pagename)
		else:
			die_with_message("Cannot delete attachment: unknown attachment name.")
	else:
		title = 'Delete ' + pagename + '\'s attachment ' + name + '?'
		stdheader(title, '<h1 class="wikipagetitle">' + title + '</h1>')
		print """
<p>Are you sure you want to delete attachment %(name)s? This action is
irreversible. If you wish to continue, check the checkbox below and
click on the "Confirm" button. Otherwise, use the "back" button of
your browser.</p>
<center>
<form method="post" action="%(scriptname)s">
<input type="hidden" name="mode" value="delattach">
<input type="hidden" name="arg" value="%(pagename)s">
<input type="hidden" name="data" value="%(name)s">
<input type="checkbox" name="confirm" value="yes">&nbsp;Confirm attachment deletion<br>
<input type="submit" value=" Confirm ">
</form>
</center>
""" % {
	'scriptname': scriptname,
	'pagename': pagename,
	'name': name
}
		stdtrailer()

###########################################################################
# Handler for 'mode=rights' - allows the user to edit the permissions set
# on a page.

def rightspage(pagename):
	checkperms(pagename, 'rights', 'viewing/editing rights')

	dummy = getarg('dummy')

	new_rights = Users.get_rights_struct()
	new_owner = getarg('owner')

	creator = dbget(pagename, 'creator', '<unknown>')
	owner = dbget(pagename, 'owner', '<unknown>')

	if dummy:
		dbset(pagename, 'rights', new_rights)

		if Users.superuser() or Users.username() in [creator, owner]:
			if new_owner and new_owner != '<unknown>':
				dbset(pagename, 'owner', new_owner)

		redirect(pagename, "Rights saved", None, 'edit')
	else:
		rights = dbget(pagename, 'rights', Config.default_rights)

		stdheader("Edit rights", '<h1 class="wikipagetitle">Edit rights</h1>')
		print '<form method="post" action="' + scriptname + '">'
		print 'Creator: ' + creator + '<br>'
		print 'Owner:',
		if Users.superuser() or Users.username() in [creator, owner]:
			print '<input type="text" name="owner" value="' + owner + '"><br>'
		else:
			print owner + '<br>'
		print '<input type="hidden" name="mode" value="rights">'
		print '<input type="hidden" name="arg" value="' + pagename + '">'
		print '<input type="hidden" name="dummy" value="dummy">'
		print '<h1 class="wikipagetitle">Rights for ' + pagename + '</h1>'
		Users.print_rights_cbs(rights)
		print '<input type="submit" name="submit" value=" Submit ">'
		print '</form>'
		stdtrailer()

###########################################################################
# Handler for 'mode=subscribe' - adds a page name to the current user's
# subscription list.

def subscribepage(pagename):
	user = Users.current()
	user.subscribe_to(pagename)
	stdheader("Subscribed", '<h1 class="wikipagetitle">Subscribed</h1>')
	print '<p>You are now subscribed to page ' + pagename + '.</p>'
	print '<p>Click ' + internal_link(pagename, 'read', 'here') + ' to return to the page.</p>'
	stdtrailer()

###########################################################################
# Handler for 'mode=unsubscribe' - removes a page name from the
# current user's subscription list.

def unsubscribepage(pagename):
	user = Users.current()
	user.unsubscribe_from(pagename)
	stdheader("Unsubscribed", '<h1 class="wikipagetitle">Unsubscribed</h1>')
	print '<p>You are no longer subscribed to page ' + pagename + '.</p>'
	print '<p>Click ' + internal_link(pagename, 'read', 'here') + ' to return to the page.</p>'
	stdtrailer()

###########################################################################
# Handler for 'mode=history' - lets a user browse a page's CVS history

def historypage(pagename):
	if not use_cvs:
		die_with_message("Not in CVS mode.")
		return

	log = db.dict.item_log(pagename + '.txt')

	if not log:
		die_with_message("No history found for page " + pagename)

	title = 'History for ' + pagename
	stdheader(title, '<h1 class="wikipagetitle">' + title + '</h1>')

	print '<table border=1 cellpadding=3>'

	headings = []
	for ent in log:
		for k in ent[1].keys():
			if k != 'author' and not k in headings:
				headings = headings + [k]

	print '<tr><th></th>'
	for h in headings:
		print '<th>' + h + '</th>'
	print '</tr>'

	top_rev = ''
	prev_rev = ''

	for ent in log:
		print '<tr><td>' + internal_link(pagename, 'read', 'Revision ' + ent[0], ent[0])

		if Config.cvsweb_path:
			if prev_rev:
				print ' <a href="' + Config.cvsweb_path + \
					pagename + '.txt.diff?r1=' + \
					ent[0] + '&r2=' + prev_rev + '">[diff to next]</a>'

			if top_rev:
				print ' <a href="' + Config.cvsweb_path + \
					pagename + '.txt.diff?r1=' + \
					ent[0] + '&r2=' + top_rev + '">[diff to top]</a>'

		prev_rev = ent[0]
		if not top_rev:
			top_rev = ent[0]

		print '</td>'
		for k in headings:
			if ent[1].has_key(k):
				print '<td>' + ent[1][k] + '</td>'
			else:
				print '<td></td>'
		print '</tr>'

	print '</table>'
	stdtrailer()
