import sys
import posix

###########################################################################
# Configuration of the Wiki
###########################################################################

# On MacOSX, you will need the gdbm python module installed. If you
# have Fink, the version of python in /sw has gdbm available, so
# uncomment this line to pick up the /sw python path:
sys.path.append('/sw/lib/python2.3/lib-dynload')

# Directory to store information in
pyledir = '.'

# If nonzero, use CVS - otherwise use gdbm.
# CVS has a revision history.
# gdbm is faster. (*much* faster.)
# CVS has fine-grained database locking.
# gdbm has very coarse database locking.
use_cvs = 0

# Filename of database to use to retrieve/store pages
#
# NOTE: If using CVS, this must be a DIRECTORY which is
# a checked-out CVS module with no files in it to start with.
# (Otherwise it should be a regular file-path.)
#
pyledb = pyledir + '/pyledb'

# If you're using CVS, and you want to use cvsweb to generate
# diffs between versions of pages, define cvsweb_path to be
# a URL fragment pointing to the relevant CVS module viewed
# within cvsweb, eg.
#
# cvsweb_path = '/cvsweb/index.cgi/pyledb/'
#
# If you don't want to use cvsweb, leave cvsweb_path = None.
#
cvsweb_path = None

# Names of template html files
templatehead = pyledir + '/template-head'
templatetail = pyledir + '/template-tail'
templateredir = pyledir + '/template-redir'

# Formatting to apply to headers at various levels
header_formatting = [
	('<h1>', '</h1>'),
	('<h2>', '</h2>'),
	('<h3>', '</h3>'),
	('<h4>', '</h4>'),
	('<h5>', '</h5>'),
	('<h6>', '</h6>')
]

# (relative or absolute) URL to logo to display
logo_url = 'pylelogo-small.png'

# Name of anonymous users
anonymous_user = "Anonymous"

# Allow anonymous editing of pages?
allow_anonymous_edit = 0

# Allow anonymous viewing of pages?
allow_anonymous_view = 1

# Allow web-based user creation for non-SU users?
allow_user_create = 1

# Default per-page access rights
default_rights = {
	'anonymous read':	1,	# can anonymous see the page?
	'anonymous write':	1,	# can anonymous edit page attributes/text?
	'anonymous rights':	0,	# can anonymous edit page permissions?
	'named read':		1,	# can named see the page?
	'named write':		1,	# can named edit page attributes/text?
	'named rights':		0	# can named edit page permissions?
}

# Name of cookie used to hold user name
user_cookie_name = 'pyleuser'

# Name of default page
frontpage = 'FrontPage'

# Internal-link pattern - be careful when changing this
#linkpattern = r'([A-Z]+[a-z]+){2,}'
linkpattern = r'\b([A-Z]+[a-z0-9]+){2,}'

# Mime types recognised for attachment uploads - feel free to change these
# Note that each array entry is a pair of MIME type and extension. The
# extension is used when generating static webtrees of a pyle database.
mimetypes = [
	('application/octet-stream', ''),
	('text/html', '.html'),
	('text/plain', '.txt'),
	('image/gif', '.gif'),
	('image/jpeg', '.jpg'),
	('image/png', '.png'),
	('application/rtf', '.rtf'),
	('application/msword', '.doc'),
	('application/x-gzip', '.gz'),
	('application/x-gzipped-tar', '.tar.gz'),
        ('application/x-zip-compressed', '.zip')
]

# Prefix of names of snapshots generated by snapshot.cgi.
# Will have '_YYYYMMDDHHMMSS.zip' appended to it.
snapshot_prefix = 'snapshot'

# Extra files to include in a snapshot ZIP file.
# Each entry is to be a tuple of real file path and ZIP-file path.
snapshot_extras = [
    ('pylelogo-small.png', 'pylelogo-small.png'),
    ('style.css', 'style.css')
]

# Pattern for mime types to inline with an HTML IMG tag rather than
# direct inclusion
inlinepattern = r'^image'

# Use "enscript" to format @code blocks? Set to None to disable.
code_enscript_command = '/usr/bin/enscript'
# code_enscript_command = None

# Size of textarea
textarea_cols = 80
textarea_rows = 25

# Set canonical_server_name to the hostname part of the main web
# address for accessing this installation of pyle. Set
# canonical_script_path to the section of the main URL locating the
# actual main pyle CGI script.
#
# These two variables are used when a user is editing pyle pages via
# FTP, where it's not obvious how to construct a correct URL.
#
canonical_server_name = 'UNKNOWN.HOST.NAME'
canonical_script_path = '/pyle/index.cgi'

# When running a standalone FTP frontend to pyle, if running as root,
# setuid to this user first:
ftp_server_user = 'apache'

# Email message to send when a subscribed page changes.
email_notification_message = \
"""%(link)s

One of the pages to which you have subscribed, %(pagename)s, has changed.
You can go to the above URL to see the new page.

If you want to see the page as it is right now, visit:
	%(link)s

If you want to see the page as it was when this message was sent, visit:
	%(versionlink)s

This message was automatically generated by PyleWiki.
""";
