# Pyle, a WikiClone in Python
# Copyright (C) 1999, 2000 Tony Garnock-Jones <tonyg@kcbbs.gen.nz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import sys
import string
import cgi
import time

class Cookie:
	def __init__(self, name, value, expiry = None, path = '/', domain = None):
		self.name = name
		self.value = value
		self.expiry = expiry
		self.path = path
		self.domain = domain

	def render(self):
		expstr = ''
		if self.expiry != None:
			expstr = '; expires=' + time.strftime('%a, %d-%b-%Y %H:%M:%S GMT', \
								time.gmtime(self.expiry))

		domstr = ''
		if self.domain:
			domstr = '; domain=' + self.domain

		return 'Set-Cookie: ' + \
			self.name + '=' + self.value + \
			expstr + \
			'; path=' + self.path + \
			domstr

class Cookies:
	def __init__(self):
		self.cookies = {}
		self.changed = {}

		if os.environ.has_key('HTTP_COOKIE'):
			cs = map(string.strip, string.split(os.environ['HTTP_COOKIE'], ';'))
			for c in cs:
				l = string.split(c, '=')
				n = l[0]
				v = l[1]
				self.cookies[n] = Cookie(n, v)

	def __len__(self):
		return len(self.cookies)

	def __getitem__(self, key):
		return self.cookies[key].value

	def __setitem__(self, key, value):
		return self.set(key, value)

	def __delitem__(self, key):
		c = self.cookies[key]
		self.changed[key] = c
		c.expiry = time.time() - 86400		# one day ago. Should be enough.

	def has_key(self, key):
		return self.cookies.has_key(key)

	def set(self, key, value, expiry = None, path = '/', domain = None):
		if self.cookies.has_key(key):
			c = self.cookies[key]
			c.value = value
			c.expiry = expiry
			c.path = path
			c.domain = domain
		else:
			c = Cookie(key, value, expiry, path, domain)
			self.cookies[key] = c
		self.changed[key] = c
		return value

	def render(self):
		return string.join(map(Cookie.render, self.cookies.values()), '\n')
