import sys
import shelve
import string
import os
import re
import cgi
import time
import posix

from threading import *

from Utils import *
import Users
import Config
import FtpServer

###########################################################################

INTERNAL_release_lock()		# because Utils gains the lock by default.

class DBguard:
	mutex = Lock()
	open = 0

	def __init__(self, activeUser):
		DBguard.mutex.acquire()
		try:
			if not DBguard.open:
				INTERNAL_acquire_lock()
			DBguard.open = DBguard.open + 1
			if activeUser:
				activeUser.activate()
		finally:
			DBguard.mutex.release()

	def __del__(self):
		DBguard.mutex.acquire()
		try:
			DBguard.open = DBguard.open - 1
			if not DBguard.open:
				INTERNAL_release_lock()
		finally:
			DBguard.mutex.release()

###########################################################################

def checkperms(pagename, mode, action = None, die = 550):
	rights = dbget(pagename, 'rights', Config.default_rights)
	message = ''

	if Users.logged_in():
		if Users.superuser(): return 1

		name = Users.username()
		if name == dbget(pagename, 'creator'): return 1
		if name == dbget(pagename, 'owner'): return 1
		if rights['named ' + mode]: return 1

		if die:
			message = message + 'Access for user ' + name + ' '
	else:
		if rights['anonymous ' + mode]: return 1

		if die:
			message = message + 'Anonymous access '

	if die:
		message = message + 'denied when trying to ' + mode
		if mode == 'read':
			message = message + ' from '
			link_mode = 'backlinks'
		else:
			message = message + ' to '
			link_mode = 'read'
		message = message + 'page ' + pagename + '.'

		if action:
			message = message + ' Action was: ' + action + '.'

		raise FtpServer.FtpResponse(die, message)

	return 0

class PyleFtpFS(FtpServer.FlatFileSystem):
	def log_action(self, action, argl):
		import time

		timestamp = time.strftime('%Y/%m/%d %H:%M:%S', time.localtime(time.time()))
		entry = string.join([timestamp, self.user.name, action, str(argl)], '\t')
		print entry

	def login_check(self, username, password):
		x = DBguard(None)
		self.user = Users.finduser(username)
		if not self.user or not self.user.login(password):
			return 0
		self.user.activate()
		self.log_action('login', [])
		return 1

	def allFilenames(self):
		x = DBguard(self.user)
		result = []
		keys = db.keys()
		keys.sort()
		for k in keys:
			if self.contains(k):
				result.append(k)
		return result

	def contains(self, filename):
		x = DBguard(self.user)
		result = dbpageexists(filename) and checkperms(filename, 'read', None, 0)
		return result

	def contentsFor(self, filename):
		x = DBguard(self.user)
		self.log_action('retr', [filename])
		result = dbgettext(filename, None)
		return result

	def lengthOf(self, filename):
		x = DBguard(self.user)
		return len(dbgettext(filename, None))

	def userFor(self, filename):
		x = DBguard(self.user)
		result = dbget(filename, 'user', 'unknown')
		return result

	def mtimeFor(self, filename):
		x = DBguard(self.user)
		result = dbget(filename, 'time', 0)
		return result

	def updateCheck(self, filename):
		if filename.endswith('~'):
			# Don't let Emacs save "backups" here.
			return 0
		x = DBguard(self.user)
		result = checkperms(filename, 'write', 'saving page', 0)
		return result

	def updateContent(self, filename, newcontent):
		x = DBguard(self.user)
		timenow = time.time()
		dbset(filename, 'time', timenow)
		dbset(filename, 'user', Users.username())
		dbset(filename, 'text', preprocess(filename, newcontent, 0))
		os.environ['REMOTE_ADDR'] = 'FTP'
		notechange(filename, 'saved', timenow)
		notify_subscribers(filename)
		self.log_action('stor', [filename])

	def deleteContent(self, filename):
		x = DBguard(self.user)
		checkperms(filename, 'write', 'deleting page')
		dbremove(filename)
		os.environ['REMOTE_ADDR'] = 'FTP'
		notechange(filename, 'removed', time.time())
		self.log_action('dele', [filename])


if __name__ == '__main__':
	if posix.getuid() == 0:
		print "Running as root, suing to " + ftp_server_user
		import pwd
		try:
			posix.setuid(pwd.getpwnam(ftp_server_user)[2])
		except KeyError:
			print "Cannot su to " + ftp_server_user + "!"

	svr = FtpServer.FtpServer(('localhost', 8021), PyleFtpFS,
				  bannerText = 'Pyle FTP service ready.')
	svr.serve_forever()
