#!/usr/bin/python

# Pyle, a WikiClone in Python
# Copyright (C) 1999, 2000 Tony Garnock-Jones <tonyg@kcbbs.gen.nz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import shelve
import string
import os
import re
import cgi
import time
import posix

from Utils import *
import Users

def _printrow(*xs):
	print '<tr>'
	for x in xs:
		print '  <td>' + x + '</td>'
	print '</tr>'

def prefspage():
	stdheader("Configuration")
	if not Users.logged_in():
		Users.send_loginform("pyle.cgi", {})
		print '<hr>'
	else:
		name = getarg('name')
		if Users.superuser() and name:
			user = Users.finduser(name)
		else:
			user = Users.current()

		print '<h1 class="wikipagetitle">Preferences for ' + user.name + '</h1>'
		print '<form method="post" action="' + scriptname + '">'
		print '<input type="hidden" name="mode" value="updprefs">'
		print '<input type="hidden" name="name" value="' + user.name + '">'

		print '<hr><h3>Email address</h3>'
		print '<p>Your email address is used to send notifications of changes made to'
		print 'pages you have subscribed to.</p>'
		print 'Email address: <input type="text" size="60" name="newemail" value="' + user.emailaddr + '"><br>'
		print '<br>'

		pagenames = user.get_subscriptions()
		print '<hr><h3>Subscribed Pages</h3>'
		if pagenames:
			print '<table border=0><tr valign="top"><td>'
			print '<select multiple size="6" width="200" name="unsubscribe">'
			for pagename in user.get_subscriptions():
				print '<option>' + pagename + '</option>'
			print '</select>'
			print '</td><td width=20></td><td>'
		print '<p>When these pages are edited by anyone except you, you will be sent'
		print 'email notifying you of the change. If you want to unsubscribe from'
		print 'any of these pages, select their names here before you click on "submit"'
		print 'below.</p>'
		if pagenames:
			print '</td></tr></table>'
		else:
			print '<p><i>You are not subscribed to any pages.</i></p>'
		print '<br>'

		print '<hr><h3>Password</h3>'
		print '<p>To change your password, enter your current password once, and your'
		print 'new password twice. If you don\'t want to change your password, leave these'
		print 'fields blank.</p>'
		print '<table border=0>'
		_printrow('Old password:', '<input type="password" name="oldpass" value="">')
		_printrow('New password:', '<input type="password" name="newpass" value="">')
		_printrow('New password (again):',
			  '<input type="password" name="newpass2" value="">')
		print '</table>'
		print '<br>'

		print '<hr><h3>Default rights</h3>'
		print '<p>These settings control the default set of permissions given to a '
		print 'page that you have just created.</p>'
		Users.print_rights_cbs(user.get_rights(), 'the')
		print '<br>'

		print '<hr>'
		print '<input type="submit" name="submit" value=" Save Changes ">'
		print '</form>'

	print '<p>'
	print '<b>Click <a href="pyle.cgi">here</a> to return to the front page.</b><br>'
	print Users.get_toolbar()
	print '</p>'
	stdtrailer()

def updprefspage():
	name = getarg('name', '')
	newemail = getarg('newemail', '')
	oldpass = getarg('oldpass')
	newpass = getarg('newpass')
	newpass2 = getarg('newpass2')
	new_rights = Users.get_rights_struct()

	if newpass != newpass2:
		die_with_message("You must enter the same password in both fields. " + \
				"Please use the back button on your browser, and try again.")

	if Users.superuser() and name:
		user = Users.finduser(name)
	else:
		user = Users.current()

	if oldpass and newpass:
		if not user.change_password(oldpass, newpass):
			die_with_message("You must enter the correct current password in " + \
					"order to change your password to something else. " + \
					"Please try again.")

	user.set_email(newemail)
	user.set_rights(new_rights)

	for pagename in getarg_list('unsubscribe'):
		user.unsubscribe_from(pagename)

	stdheader("User information changed")
	print '<h1 class="wikipagetitle">Preferences updated</h1>'
	print '<p>Click <a href="' + scriptname + '?name=' + name + '">here</a> to see'
	print 'your changes.</p>'
	stdtrailer()

def logoutpage():
	response = 'You have been logged out.'
	if Users.logged_in():
		del(cookies[user_cookie_name])
	else:
		response = response + ' (You weren\'t logged in.)'
	stdheader('Logged Out')
	print '<h1 class="wikipagetitle">' + response + '</h1>'
	print '<p>Click <a href="pyle.cgi">here</a> to return to the front page.</p>'
	stdtrailer()

def createuserpage():
	newname = getarg('newname')
	newemail = getarg('newemail', '')
	newpass = getarg('newpass')
	newpass2 = getarg('newpass2')

	if not newname:
		stdheader('Create new user')
		print '<p>Please fill in the form below and click on "submit" to'
		print 'create a new user:</p>'
		print '<form method="post" action="' + scriptname + '">'
		print '<table border=0>'
		_printrow('New user name:', '<input type="text" name="newname" value="">')
		_printrow('Email address:',
			  '<input type="text" size="60" name="newemail" value="">')
		_printrow('&nbsp;')
		_printrow('New Password:', '<input type="password" name="newpass" value="">')
		_printrow('Confirm Password:', '<input type="password" name="newpass2" value="">')
		print '</table>'
		print '<input type="hidden" name="mode" value="create">'
		print '<br>'
		print '<input type="submit" name="submit" value=" Submit "><br>'
		print '</form>'
		stdtrailer()
		return

	if not newpass:
		die_with_message('You must choose a password. Please go back and try again.')

	if newpass != newpass2:
		die_with_message('You must enter the same password in both password fields ' + \
				'in order to create a new user. Click the back button on ' + \
				'you browser, and try again.')

	if Users.finduser(newname) or (newname == anonymous_user):
		die_with_message('A user with that name already exists. Please try again ' + \
				'with a different username.')

	user = Users.User(newname, newemail, newpass)
	if not Users.logged_in():
		Users.process_auth(newname, newpass)

	stdheader("User created")
	print '<h1 class="wikipagetitle">User ' + user.name + ' created</h1>'
	print 'Click <a href="' + scriptname + '">here</a> to continue.'
	stdtrailer()

def deluserpage():
	name = getarg('name')
	conf = getarg('conf', 'no')

	if not name:
		stdheader("Delete user", '<h1 class="wikipagetitle">Delete User</h1>')
		print '<form method="post" action="' + scriptname + '">'
		print '<input type="hidden" name="mode" value="deluser">'
		print 'User name to delete: <input type="text" name="name" value=""><br>'
		print '<input type="submit" name="submit" value=" Submit ">'
		stdtrailer()
		return

	if conf != 'yes':
		stdheader("Confirm user deletion",
			  '<h1 class="wikipagetitle">Confirm user deletion</h1>')
		print '<form method="post" action="' + scriptname + '">'
		print '<input type="hidden" name="mode" value="deluser">'
		print '<input type="hidden" name="name" value="' + name + '">'
		print '<input type="checkbox" name="conf" value="yes">&nbsp;Confirm user deletion<br>'
		print '<input type="submit" name="submit" value=" Submit ">'
		stdtrailer()
		return

	user = Users.finduser(name)
	if not user:
		die_with_message("User not found - no user removed.")

	user.remove()
	stdheader("User deleted", '<h1 class="wikipagetitle">User deleted</h1>')
	print 'Click <a href="' + scriptname + '">here</a> to continue.'
	stdtrailer()

def listuserpage():
	dummy = getarg('dummy')
	name = getarg('name', '')

	stdheader("List users", '<h1 class="wikipagetitle">List users</h1>')
	print '<form method="post" action="' + scriptname + '">'
	print '<input type="hidden" name="mode" value="listuser">'
	print '<input type="hidden" name="dummy" value="dummy">'
	print 'User name regexp to search for: <input type="text" name="name" value="">'
	print '<input type="submit" name="submit" value=" Submit ">'
	print '</form>'

	if dummy:
		print '<hr><h1 class="wikipagetitle">Search results</h1>'

		lst = Users.find_users_matching(re.compile(name))
		if not lst:
			print '<i>No matching usernames found.</i>'
		else:
			print '<table border="1"><tr align="left"><th>Username</th>'
			print '<th>Email address</th>'
			print '<th>Last login</th>'
			print '<th>SU?</th>'
			print '</tr>'
			for u in lst:
				print '<tr align="left"><td>'
				print '<a href="' + scriptname + '?name=' + u.name + '">' 
				print u.name + '</a>'
				print '</td><td>' + u.get_email()
				print '</td><td>',
				lastlogin = u.get_last_login()
				if lastlogin:
					print time.strftime("%d %B %Y, %H:%M GMT", \
							time.gmtime(u.get_last_login()))
				else:
					print 'never'
				print '</td><td>',
				if u.get_superuser():
					print 'X'
				print '</td></tr>'
			print '</table>'

	stdtrailer()
