# Pyle, a WikiClone in Python
# Copyright (C) 1999, 2000 Tony Garnock-Jones <tonyg@kcbbs.gen.nz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import os
import sys
import string
import cgi
import time
import re

from Utils import *
import Config

###########################################################################
loaded_users = {}

###########################################################################
class User:
	def __init__(self, name, emailaddr, password):
		self.name = name
		self.emailaddr = emailaddr
		self.password = password
		self.last_login = None
		self.superuser = 0
		self.subscriptions = []
		self.store()

	def login(self, loginpass):
		if self.password != loginpass:
			return 0
		self.last_login = time.time()
		self.store()
		return 1

	def get_last_login(self):
		return self.last_login

	def activate(self):
		activateuser(self.name)

	def change_password(self, initialpass, newpass):
		if self.password != initialpass:
			return 0
		self.password = newpass
		self.store()
		return 1

	def get_email(self):
		return self.emailaddr

	def set_email(self, newaddr):
		self.emailaddr = newaddr
		self.store()

	def get_superuser(self):
		return self.superuser

	def set_superuser(self, su):
		self.superuser = su
		self.store()

	def get_rights(self):
		if hasattr(self, 'rights'):
			return self.rights
		else:
			return Config.default_rights

	def set_rights(self, rights):
		self.rights = rights
		self.store()

	def get_subscriptions(self):
		if not hasattr(self, 'subscriptions'):
			self.subscriptions = []
			self.store()
		return self.subscriptions

	def is_subscribed_to(self, pagename):
		return pagename in self.get_subscriptions()

	def subscribe_to(self, pagename):
		old_subs = self.get_subscriptions()
		if not pagename in old_subs:
			self.subscriptions = old_subs + [pagename]
		self.store()

	def unsubscribe_from(self, pagename):
		self.subscriptions = filter(lambda x, pagename=pagename: x != pagename,
					self.get_subscriptions())
		self.store()

	def remove(self):
		users = dbget('users', 'users', {})
		del(users[self.name])
		dbset('users', 'users', users)
		del(loaded_users[self.name])

	def store(self):
		users = dbget('users', 'users', {})
		users[self.name] = self
		dbset('users', 'users', users)
		loaded_users[self.name] = self

###########################################################################
username_v = None
currentuser = None

def finduser(loginname):
	if not loaded_users.has_key(loginname):
		users = dbget('users', 'users', {})

		if not users.has_key(loginname):
			return None

		loaded_users[loginname] = users[loginname]

	return loaded_users[loginname]

def all_users():
	for user in dbget('users', 'users', {}).values():
		username = user.name
		if not loaded_users.has_key(username):
			loaded_users[username] = user
	return loaded_users.values()

def activateuser(u):
	global username_v
	global currentuser

	username_v = u
	currentuser = finduser(username_v)

def username():
	global username_v
	return username_v

def current():
	global currentuser
	return currentuser

def superuser():
	return currentuser and currentuser.get_superuser()

def default_rights():
	if currentuser:
		return currentuser.get_rights()
	else:
		return Config.default_rights

###########################################################################
# Discover name of the user
if os.environ.has_key('REMOTE_USER'): activateuser(os.environ['REMOTE_USER'])
elif cookies.has_key(user_cookie_name): activateuser(cookies[user_cookie_name])
if not username_v:
	username_v = anonymous_user

###########################################################################
# process_auth()
#
# Processes any attempts at authorisation.
# Validates authentication details and logs the user in.

def process_auth(loginname, loginpass):
	if loginname:
		if validate(loginname, loginpass):
			cookies[user_cookie_name] = loginname
			activateuser(loginname)

###########################################################################
# ensure_auth()
#
# Makes sure that the user is logged in correctly

def ensure_auth():
	if not logged_in():
		send_loginpage()
		sys.exit(0)

###########################################################################
# logged_in()
#
# Returns true if the user is currently logged in, or false if a login
# screen needs to be presented.

def logged_in():
	if current():
		return 1
	return 0

###########################################################################
# get_toolbar()
#
# Returns an HTML fragment for the user-administration toolbar

def get_toolbar():
	result = '<a href="user.cgi">'
	if logged_in():
		result = result + 'User Options for <b>' + username() + '</b>' \
				+ '</a>&nbsp;|&nbsp;<a href="user.cgi?mode=logout">Log out'
	else:
		result = result + 'Log in'
	result = result + '</a>'
	if superuser():
	    result = result + '&nbsp;|&nbsp;<a href="user.cgi?mode=create">Create new user</a>' \
				+ '&nbsp;|&nbsp;<a href="user.cgi?mode=deluser">Remove user</a>' \
				+ '&nbsp;|&nbsp;<a href="user.cgi?mode=listuser">List users</a>'
	return result

###########################################################################
# send_loginpage()
# send_loginform()
#
# Sends a login page to the user, requesting login...

def send_loginpage():
	stdheader('Login', '<h1 class="wikipagetitle">Login</h1>')
	send_loginform()
	stdtrailer()

def make_hidden_input(name, value):
	return '<input type="hidden" name="' + name + '" value="' + htmlParamQuote(value) + '">'

def send_loginform(action = None, args = None):
	if not action:
		action = scriptname

	print '<form method="post" action="' + action + '">'
	print '<p>To log in to the system, please complete the form below:</p>'
	print '<table border=0>'
	print '<tr><td>User name:</td><td><input type="text" name="loginname" value=""></td></tr>'
	print '<tr><td>Password:</td>' + \
	      '<td><input type="password" name="loginpass" value=""></td></tr>'
	print '</table>'
	print '<br>'
	print '<input type="submit" name="submit" value=" Submit "><br>'
	print '<input type="hidden" name="force-unquote" value="yes">'
	if args:
		for k in args.keys():
			print make_hidden_input(k, args[k])
	else:
		for k in form.keys():
			print make_hidden_input(k, form[k].value)
	if Config.allow_user_create or superuser():
		print '<p>If you do not have a user account, please'
		print '   <a href="user.cgi?mode=create">click here to create one</a>.</p>'
	print '</form>'

def validate(loginname, loginpass):
	user = finduser(loginname)
	if not user or not user.login(loginpass):
		die_with_message('Invalid login. Please use the back button on your browser ' + \
				'and try again.')
	return 1

def find_users_matching(rexp):
	users = dbget('users', 'users', {})
	lst = filter(lambda k, rexp=rexp: rexp.search(k), users.keys())
	lst = map(lambda k, users=users: users[k], lst)
	return lst

def get_rights_struct():
	anon_r = getbool('anon_r')
	anon_w = getbool('anon_w')
	anon_p = getbool('anon_p')
	named_r = getbool('named_r')
	named_w = getbool('named_w')
	named_p = getbool('named_p')

	return {
		'anonymous read':	anon_r,
		'anonymous write':	anon_w,
		'anonymous rights':	anon_p,
		'named read':		named_r,
		'named write':		named_w,
		'named rights':		named_p
	}

def print_rights_cbs(rights, word = 'this'):
	def make_cb(name, defkey, title, rights=rights, word=word):
		print '<input type="checkbox" name="' + name + '"',
		if rights[defkey]:
			print ' checked',
		print 'value="yes">&nbsp;' + title + ' ' + word + ' page<br>'

	print '<table border="1" cellpadding="10" width="100%"><tr align="left" valign="top">'
	print '<td>'
	make_cb('anon_r', 'anonymous read', "Allow anonymous users to read")
	make_cb('anon_w', 'anonymous write', "Allow anonymous users to edit or " + \
						"delete the contents of")
	make_cb('anon_p', 'anonymous rights', "Allow anonymous users to alter " + \
						"the rights for")
	print '</td><td>'
	make_cb('named_r', 'named read', "Allow logged-in users to read")
	make_cb('named_w', 'named write', "Allow logged-in users to edit or " + \
						"delete the contents of")
	make_cb('named_p', 'named rights', "Allow logged-in users to alter " + \
						"the rights for")
	print '</td></tr></table>'
