# Pyle, a WikiClone in Python
# Copyright (C) 1999, 2000 Tony Garnock-Jones <tonyg@kcbbs.gen.nz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

from __future__ import nested_scopes

import sys
import shelve
import string
import os
import re
import cgi
import time
import traceback
import exceptions

# These imports are run later - only if we're using the respective modules
#import cvsdb
#import gdbm

from Config import *
import Cookie

import Block
import sublanguages

###########################################################################
# Cookie processing

cookies = Cookie.Cookies()

###########################################################################
# Automated setup/configuration

# Name of the script itself
if os.environ.has_key('SCRIPT_NAME'):
	scriptname = os.environ['SCRIPT_NAME']
else:
	scriptname = canonical_script_path

# Name of the webserver-host we are working with
if os.environ.has_key('SERVER_NAME'):
	servername = os.environ['SERVER_NAME']
else:
	servername = canonical_server_name

# See definition of render() below for REs used to format text

###########################################################################
# Database access routines

def INTERNAL_fresh_gdbm():
	while 1:
		try:
			result = gdbm.open(pyledb, 'w', 0666)
			return result
		except gdbm.error:
			time.sleep(1)
			continue

if use_cvs:
	import cvsdb
	internal_db = cvsdb.open(pyledb, '')
	internal_db.exclude_cvs('users')
else:
	import gdbm
	internal_db = INTERNAL_fresh_gdbm()

db = shelve.BsdDbShelf(internal_db)
form = cgi.FieldStorage()

def INTERNAL_release_lock():
	db.sync()
	if not use_cvs:
		db.close()

def INTERNAL_acquire_lock():
	if not use_cvs:
		db.dict = INTERNAL_fresh_gdbm()

def htmlParamQuote(str):
	str = string.replace(str, '%', '%25')
	str = string.replace(str, '"', '%22')
	str = string.replace(str, ' ', '%20')
	return str

force_unquote = form.has_key('force-unquote')

def htmlParamUnquote(str):
	if force_unquote:
		str = string.replace(str, '%20', ' ')
		str = string.replace(str, '%22', '"')
		str = string.replace(str, '%25', '%')
	return str

def getarg(argname, default = None):
	if form.has_key(argname):
		entry = form[argname]
		if type(entry) is type([]):
			return map(lambda x: htmlParamUnquote(x.value), entry)
		else:
			return htmlParamUnquote(entry.value)
	return default

def getarg_list(argname, default = []):
	entry = getarg(argname)
	if entry:
		if type(entry) is type([]):
			return entry
		else:
			return [entry]
	else:
		return default

def getbool(argname):
	if getarg(argname, 'no') == 'yes':
		return 1
	return 0

def dbget(pagename, attrib, defvalue = None, version = None):
	try:
		if use_cvs and attrib == 'text':
			if version:
				return db.dict.get_version(pagename + '.txt', version)
			else:
				return db.dict[pagename + '.txt']

		entry = db[pagename]
		return entry[attrib]
	except:
		pass

	return defvalue

def dbset(pagename, attrib, value):
	try:
		db[pagename]
	except:
		import Users
		db[pagename] = { 'creator': Users.username(),
				 'owner': Users.username(),
				 'rights': Users.default_rights() }

	if use_cvs and attrib == 'text':
		db.dict[pagename + '.txt'] = value
	else:
		entry = db[pagename]
		entry[attrib] = value
		db[pagename] = entry

def dbclear(pagename, attrib):
	if use_cvs and attrib == 'text' and db.has_key(pagename + '.txt'):
		del db.dict[pagename + '.txt']
	elif db.has_key(pagename):
		entry = db[pagename]
		if entry.has_key(attrib):
			del entry[attrib]
			db[pagename] = entry

def dbremove(pagename):
	if db.has_key(pagename):
		del db[pagename]
	if use_cvs and db.has_key(pagename + '.txt'):
		del db.dict[pagename + '.txt']

def dbgettext(pagename, version = None):
	return dbget(pagename, 'text', 'Describe ' + pagename + ' here.', version)

def dbgettextversion(pagename):
	return db.dict.current_version(pagename + '.txt')

def dbpageexists(pagename):
	return db.has_key(pagename) \
		and pagename != 'users' \
		and pagename[-4:] != '.txt'

###########################################################################
# mimetype_extension - returns the extension to use for a particular mime
# type.

mimetype_map = {}
for (k, v) in mimetypes:
	mimetype_map[k] = v

def mimetype_extension(t):
	if mimetype_map.has_key(t):
		return mimetype_map[t]
	else:
		return ''

###########################################################################
# internal_link - returns HTML text for linking to another page within this
# CGI.
#
#	pagename	the name of the WikiPage to link to
#	service		what to do with the page
#	vistext		the text the user sees - defaults to /pagename/
#	data		optional argument to the new page - defaults to None
#
# Slightly modified now to link to static copies if being used to generate
# a static webtree:
#
# static_mode() - true if we are generating a static webtree, false for the
# (normal) dynamic case.
#
# set_static_mode(newmode) - allows change of mode on-the-fly.

in_static_mode = 0
def static_mode():
	global in_static_mode
	return in_static_mode

def set_static_mode(n):
	global in_static_mode
	in_static_mode = n

	if not vars().has_key('scriptname'):
		global scriptname
		scriptname = 'pyle.cgi'		# Just a guess. Placeholder.

def internal_link(pagename, service = 'read', vistext = None, data = None):
	if not vistext:
		vistext = pagename

	if data:
		datatext = '&data=' + data
	else:
		datatext = ""

	if static_mode():
		if dbpageexists(pagename):
			atts = dbget(pagename, 'attachments', {})
			if service == 'read':
				return '<a class="wikilink" href="' + \
				       pagename + '.html">' + vistext + '</a>'
			elif service == 'getattach' and atts.has_key(data):
				att = atts[data]
				return '<a class="attachment" href="' + pagename + \
				       '_getattach_' + data + \
					mimetype_extension(att.mimetype) + '">' + vistext + '</a>'
			elif service in ['backlinks']:
				if data:
					data = '_' + data
				else:
					data = ''
				return '<a class="backlinks" href="' + pagename + \
				       '_' + service + data + '.html">' + vistext + '</a>'
		return vistext
	else:
		if service == 'read' and not dbpageexists(pagename) and not data:
			return '<a class="wikiword" href="' + scriptname + \
			       '?mode=edit&arg=' + pagename + \
			       datatext + '">' + vistext + '</a>'
		else:
			return '<a class="wikilink" href="' + scriptname + \
			       '?mode=' + service + '&arg=' + pagename + \
			       datatext + '">' + vistext + '</a>'

###########################################################################
# stdheader and stdtrailer - routines for sending standard HTTP response
# headers and the contents of the template-files for page header and footer.
#
# stdheader:
#	title		the HTML title of the page to display
#	heading		text to be substituted within the template
#			- defaults to ''. Often used for an HTML H1 element.

def stdheader(title, heading = ''):
	if not static_mode():
		print "Content-type: text/html"
		print cookies.render()
		print

	f = open(templatehead, "r")
	text = f.read()
	f.close()

	print text % {
		'title': title,
		'heading': heading,
		'logo': logo_url
	}

def stdtrailer():
	f = open(templatetail, "r")
	text = f.read()
	f.close()
	print text % {
		'logo': logo_url
	}

	if not static_mode():
		db.close()

###########################################################################
# redirect(pagename, action)
#
# Sends an HTML redirect (note! not an HTTP redirect) to the WikiPage
# /pagename/, describing the reason for redirection in /action/.

def redirect(pagename, action, url = None, mode = 'read'):
	f = open(templateredir, "r")
	text = f.read()
	f.close()

	if not url:
		url = scriptname + '?mode=' + mode + '&arg=' + pagename

	print "Content-type: text/html"
	print
	print text % {
		'redir': url,
		'pagename': pagename,
		'action': action,
		'logo': logo_url
	}

###########################################################################
# die_with_message(msg)
#
# Sends an error page to the user, with an arbitrary HTML message in it.

def die_with_message(msg):
	import Users

	stdheader('Error')
	print '<h1 class="wikipagetitle">Error</h1>'
	print msg
	print '<br>' + Users.get_toolbar()
	stdtrailer()
	sys.exit(0)

###########################################################################
# Rendering of formatted text

# regexps for matching bits of WikiFormatting

intlinkre = re.compile('(' + linkpattern + ')')
extlinkre = re.compile(r'\b((http|https|ftp|mailto|gopher|news|nntp|file):[^\s\(\)]+[^\s\(\).,\'\"?])([\s.,\'\"?]?)')
italre = re.compile(r"''(.*?)''", re.S)
boldre = re.compile(r"\*(.*?)\*", re.S)
underre = re.compile(r"\b_(.*?)_\b", re.S)

def find_plugin(category, name, entrypoint):
	try:
		mod = __import__(category + '.' + name)
		if hasattr(mod, name):
			mod = getattr(mod, name)
			if hasattr(mod, entrypoint):
				return (None, getattr(mod, entrypoint))
			else:
				return ('Plugin ' + category + '.' + name + \
					' missing entrypoint ' + entrypoint, None)
		else:
			return ('Plugin ' + category + '.' + name + \
				' did not load correctly', None)
	except exceptions.ImportError:
		return ('Could not find plugin ' + category + '.' + name, None)
	except:
		return ('Error loading plugin ' + category + '.' + name + ':\n<tt>' +
			cgi.escape(''.join(traceback.format_exception(*sys.exc_info()))) +
			'</tt>', None)

def nearest_match(text, matches):
	smallestname = None
	smallest = None
	for (matchname, substring) in matches:
		pos = text.find(substring)
		if pos != -1:
			if smallest is None or pos < smallest:
				smallestname = matchname
				smallest = pos
	return (smallestname, smallest)

class PylePageRenderer:
	def __init__(self, text, pagename, args):
		self.text = text
		self.pagename = pagename
		self.args = args

		self.attachments = dbget(pagename, 'attachments', {})

		self.postprocessors = []

		self.html_acc = []
		self.sectionstack = []
		self.toc = []

	def _appendRegexMarkup(self, s, result):
		while s:
			m = extlinkre.search(s)
			if m:
				frag = s[:m.start()]
				splice = '<a class="extlink" href="' + \
					 htmlParamQuote(m.group(1)) + '">' + \
					 cgi.escape(m.group(1)) + '</a>'
				s = m.group(3) + s[m.end():]
			else:
				frag = s
				splice = ''
				s = ''
				
			frag = cgi.escape(frag)
			frag = boldre.sub(r'<b>\1</b>', frag)
			frag = italre.sub(r'<i>\1</i>', frag)
			frag = underre.sub(r'<u>\1</u>', frag)
			frag = intlinkre.sub(lambda m: internal_link(m.group()), frag)

			result.append(frag)
			result.append(splice)

	def _collectFragment(self, s):
		frag = []
		while 1:
			(matchname, min_i) = nearest_match(s, [('backslash', '\\'),
							       ('open', '['),
							       ('close', ']')])
			if matchname:
				frag.append(s[:min_i])
				s = s[min_i:]
			else:
				frag.append(s)
				s = ''

			if not s:
				return (''.join(frag), None, '')

			if s[0] != '\\':
				return (''.join(frag), s[0], s[1:])
				
			if len(s) > 1:
				if s[1] in '[]':
					frag.append(s[1])
					s = s[2:]
				else:
					frag.append(s[0])
					s = s[1:]
			else:
				s = ''

	def collectSpan(self, s):
		nesting = 0
		result = []
		while s:
			(frag, what, s) = self._collectFragment(s)
			result.append(frag)
			if what is None:
				break
			elif what == '[':
				result.append('[')
				nesting = nesting + 1
			elif what == ']':
				if nesting == 0:
					break
				else:
					result.append(']')
					nesting = nesting - 1
			else:
				raise 'Invalid result from _collectFragment', what
		return (''.join(result), s)

	def discardSpan(self, s):
		(text, s) = self.collectSpan(s)
		return s

	def appendMarkup(self, s, result = None):
		if result is None:
			result = self.html_acc

		while s:
			(frag, what, rest) = self._collectFragment(s)
			self._appendRegexMarkup(frag, result)
			if what is None or what == ']':
				return rest
			elif what == '[':
				(name, pos) = nearest_match(rest, [('spacepos', ' '),
								   ('spacepos', '\n'),
								   ('spacepos', '\r'),
								   ('spacepos', '\t'),
								   ('closepos', ']')])
				if name == 'spacepos':
					handlername = rest[:pos]
					args = rest[pos + 1:]
				elif name == 'closepos':
					handlername = rest[:pos]
					args = rest[pos:]
				else:
					handlername = rest
					args = ''

				if not handlername or not handlername[0].isalpha():
					handlername = 'invalid_spanhandler_name'
					args = rest

				(err, plugin) = find_plugin('spanhandlers', handlername,
							    'SpanHandler')
				if plugin:
					s = plugin(args, self, result)
				else:
					result.append('<span class="missingspanhandler">' + \
						      err + '</span> ')
					s = args
			else:
				raise 'Invalid result from _collectFragment', what

	def appendHtml(self, s):
		self.html_acc.append(s)

	def appendPlain(self, s):
		self.appendHtml(cgi.escape(s))

	def addPostProcessor(self, ordering, fn):
		self.postprocessors.append((ordering, fn))

	def finish(self):
		def toc_pp(renderer):
			if self.toc:
				tocacc = ['<div class="toc"><a name="toc"></a>' + \
					  '<h4 class="toctitle">Table of Contents</h4>\n']
				lastn = 0
				for (n, anchor, line) in self.toc:
					while n > lastn: tocacc.append('<ul>'); lastn = lastn + 1
					while n < lastn: tocacc.append('</ul>'); lastn = lastn - 1
					lastn = n
					tocacc.append('<li><a href="#' + anchor + '">' + line + '</a>\n')
				while lastn > 0: tocacc.append('</ul>'); lastn = lastn - 1
				tocacc.append('</div>')
				self.appendHtml('<p><a class="toclink" href="#toc">' + \
						'Go to Table of Contents</a></p>')
				tocacc.extend(self.html_acc)
				self.html_acc = tocacc

		self.addPostProcessor(100, toc_pp)
		self.postprocessors.sort(lambda (x, f1), (y, f2): x - y)
		for (ordering, pp) in self.postprocessors:
			pp(self)

		for frag in self.html_acc: sys.stdout.write(frag)

	def begin_list(self, is_ordered):
		if is_ordered:
			self.appendHtml('<ol>')
		else:
			self.appendHtml('<ul>')

	def visit_item(self, para):
		self.appendHtml('<li>')
		self.appendMarkup(para.as_string())

	def end_list(self, is_ordered):
		if is_ordered:
			self.appendHtml('</ol>')
		else:
			self.appendHtml('</ul>')

	def visit_section(self, rank, titleline, doc):
		stack = self.sectionstack
		while len(stack) > rank:
			stack.pop()
		while len(stack) < rank:
			stack.append(0)
		stack[-1] = stack[-1] + 1

		titlemarkup = []
		self.appendMarkup(titleline, titlemarkup)
		titlemarkup = ''.join(titlemarkup)

		sectionnumber = '.'.join(map(str, stack))
		anchor = 'section_' + string.join(map(str, stack), '_')
		self.toc.append((rank, anchor, sectionnumber + ' ' + cgi.escape(titleline)))

		if rank > len(header_formatting):
			headernum = len(header_formatting) - 1
		else:
			headernum = rank - 1

		self.appendHtml('<a name="' + anchor + '"></a>' + \
				header_formatting[headernum][0] + \
				sectionnumber + ' ' + titlemarkup + \
				header_formatting[headernum][1])

	def visit_separator(self):
		self.appendHtml('<hr>')

	def visit_sublanguage(self, commandline, doc):
		commandparts = commandline.split(' ', 1)
		command = commandparts[0]
		if len(commandparts) > 1:
			args = commandparts[1]
		else:
			args = ''
		(err, plugin) = find_plugin('sublanguages', command, 'SublanguageHandler')
		if plugin:
			plugin(args, doc, self)
		else:
			self.appendHtml('<div class="missingsublanguage">' + \
					err + '</div>')

	def visit_normal(self, para):
		self.appendHtml('<p>')
		self.appendMarkup(para.as_string())

def render(text, pagename, args = None):
	if not args:
		args = {}

	renderer = PylePageRenderer(text, pagename, args)
	Block.BasicWikiMarkup(renderer).visit(Block.parsestring(text).children)
	renderer.finish()

###########################################################################
# getbacklinks(pagename)
#
# Searches the database for links to a given WikiPage

def getbacklinks(pagename):
	lst = filter(lambda lp, p=pagename: dbpageexists(lp) and string.find(dbgettext(lp), p) != -1,
		db.keys())
	lst.sort()
	return lst

###########################################################################
# notechange(pagename, action, timenow)
#
# Records on RecentChanges the fact that /pagename/ was changed, in
# the manner described by /action/ (eg. 'saved', 'removed') at
# /timenow/.
#
#	pagename	the name of the changed WikiPage
#	action		what happened to the page - one word, usually
#	timenow		the time the change took place (eg. value of time.time())

def notechange(pagename, action, timenow):
	import Users

	text = dbgettext('RecentChanges')

	datestr = time.strftime('Changes for %d %B %Y:', time.gmtime(timenow))
	timestr = time.strftime('%H:%M GMT', time.gmtime(timenow))

	pos = string.find(text, datestr)
	if pos != -1:
		pos = pos + len(datestr)
	else:
		text = text + "\n" + datestr + "\n"
		pos = len(text)

	newenttext = '\t- ' + pagename + ': - - - (' + action + ') ' + \
		Users.username() + '@' + os.environ['REMOTE_ADDR'] + ' - - - ' + \
		timestr

	oldentre = re.compile(r'^\t- ' + pagename + r'.*$', re.M)
	oldent = oldentre.search(text, pos)
	if oldent:
		text = text[:oldent.start()] + newenttext + text[oldent.end():]
	else:
		text = text + newenttext + '\n'

	dbset('RecentChanges', 'time', timenow)
	dbset('RecentChanges', 'user', Users.username())
	dbset('RecentChanges', 'text', text)

###########################################################################
# preprocess(pagename, text, convertspaces)
#
# Massages text submitted to 'mode=save' to conform to internal conventions.
# It is here than expansion of the [TimeNow] macro takes place.
# Newlines are standardised on '\n' being the line separator.
# If /convertspaces/ is true, converts all instances of 8 consecutive spaces
# to a tab character.
#
#	pagename	the WikiPage that's about to have its text replaced
#	text		the text to be massaged before saving
#	convertspaces	a boolean, determining whether 8-spaces are replaced
#			by tabs

def preprocess(pagename, text, convertspaces):
	text = string.replace(text, '\r\n', '\n')	# cope with CRLF
	text = string.replace(text, '\r', '\n')		# cope with CR (MacOS)

	if convertspaces:
		# Convert spaces to tabs
		text = string.replace(text, '        ', '\t')

	timenowstr = time.strftime("''%d %b %Y %H:%M GMT''", time.gmtime(time.time()))
	text = string.replace(text, '[TimeNow]', timenowstr)

	return text

###########################################################################
# send_email(subject, emailaddr, bodytext)
#
# Uses sendmail(1) to email a message with subject /subject/ to
# address /emailaddr/, containing body text /bodytext/.

def send_email(subject, emailaddr, bodytext):
	sendmail = os.popen("/usr/lib/sendmail -t", "w")

	if sendmail:
		sendmail.write( \
			"From: pylewiki-daemon\n" + \
			"To: " + emailaddr + "\n" + \
			"Subject: " + subject + "\n" + \
			"\n" + \
			bodytext)
		sendmail.close()

###########################################################################
# notify_subscribers(pagename)
#
# Uses send_email to notify users that a page they have subscribed to
# has been altered.

def notify_subscribers(pagename):
	import Users

	if use_cvs:
		db.sync()
		versionlink = '&data=' + dbgettextversion(pagename)
	else:
		versionlink = ''

	current = Users.current()
	for user in Users.all_users():
		if user != current and user.is_subscribed_to(pagename):
			email = user.get_email()
			linktext = 'http://' + servername + scriptname + \
				'?mode=read&arg=' + pagename

			send_email('PyleWiki at ' + servername + ': subscribed page "' + \
				pagename + '" has changed',
				email,
				email_notification_message % {
					'email': email,
					'pagename': pagename,
					'link': linktext,
					'versionlink': linktext + versionlink
				})
