#!/usr/bin/python

# Pyle, a WikiClone in Python
# Copyright (C) 2004 Tony Garnock-Jones <tonyg@kcbbs.gen.nz>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

import sys
import time
import StringIO
import zipfile

import Config
import Utils
import Users
import Actions

Utils.set_static_mode(1)

def capture_output(thunk):
    oldout = sys.stdout
    cap = StringIO.StringIO()
    sys.stdout = cap
    thunk()
    result = cap.getvalue()
    cap.close()
    sys.stdout = oldout
    return result

snapname = Config.snapshot_prefix + '_' + time.strftime('%Y%m%d%H%M%S', time.localtime())
snapfile = StringIO.StringIO()
snapshot = zipfile.ZipFile(snapfile, 'w', zipfile.ZIP_DEFLATED)

def redirect_output(path, thunk):
    output = capture_output(thunk)
    snapshot.writestr(path, output)

def checkreadable(pagename):
	rights = Utils.dbget(pagename, 'rights', Config.default_rights)

	if Users.logged_in():
		if Users.superuser(): return 1

		name = Users.username()
		if name == Utils.dbget(pagename, 'creator'): return 1
		if name == Utils.dbget(pagename, 'owner'): return 1
		if rights['named read']: return 1
	else:
		if rights['anonymous read']: return 1
	return 0

for page in Utils.db.keys():
    if Utils.dbpageexists(page):        # check for 'users' pseudopage
        if checkreadable(page):         # check that logged-in-user can read this page
            redirect_output(snapname + '/' + page + '.html', lambda: Actions.readpage(page))
            redirect_output(snapname + '/' + page + '_backlinks.html', \
                            lambda: Actions.backlinks(page))

            for (name,att) in Utils.dbget(page, 'attachments', {}).items():
                fname = snapname + '/' + page + '_getattach_' + name + \
                        Utils.mimetype_extension(att.mimetype)
                redirect_output(fname, lambda: sys.stdout.write(att.get_content()))

for (sourcefilename, targetfilename) in Config.snapshot_extras:
    snapshot.write(sourcefilename, snapname + '/' + targetfilename)

snapshot.close()
snapdata = snapfile.getvalue()
snapfile.close()

sys.stdout.write('Content-type: application/x-zip-compressed\r\n')
sys.stdout.write('Content-length: ' + str(len(snapdata)) + '\r\n')
sys.stdout.write('Content-disposition: attachment; filename="' + \
                 snapname + '.zip"\r\n')
sys.stdout.write('\r\n')

sys.stdout.write(snapdata)
